import * as React from 'react';

export interface Value {
  text: string;
  value?: string | number;
}
export type CallBackOnClick = (event: React.MouseEvent<HTMLDivElement>, value: Value[], index: number) => void;
// type ChildrenOnClick = (event: React.MouseEvent<HTMLDivElement>, currentValue: Value, index: number) => void;

export interface Base {
  /** 是否禁用 */
  disabled?: boolean;
  /** 是否多选 */
  multiple?: boolean;
}

interface Props extends Base {
  data: any[];
  value: Value[] | undefined;
  /** item是data遍历的项, index是遍历的索引，value是选中的值 */
  children: (item: any, index: number, value: Value[] | undefined, onClick: CallBackOnClick) => JSX.Element;
  /** value是选中的值，index是选中的索引 */
  onClick: CallBackOnClick;
}

class Choice extends React.Component<Props> {
  constructor(props: Props) {
    super(props);

    this.onClick = this.onClick.bind(this);
  }

   /** 格式化value */
   private formatValue(newValue: Value): Value[] {
    const { multiple, value } = this.props;

    if (multiple) {
      if (value) {
        const index = value.findIndex((item) => {
          const itemRealValue = item.value !== undefined ? item.value : item.text;
          const realNewValue = newValue.value !== undefined ? newValue.value : newValue.text;

          return itemRealValue.toString() === realNewValue.toString();
        });
        if (index >= 0) {
          value.splice(index, 1);
        } else {
          value.push(newValue);
        }

        return value;
      }
    } else {
      return [newValue];
    }

    return [];
  }

  private onClick(event: React.MouseEvent<HTMLDivElement>, newValue: Value[], index: number): void {
    const { onClick, disabled, multiple, value } = this.props;

    if (disabled) {
      return;
    }

    // 防止重复点击
    if (multiple !== true && value && value[0].text === newValue[0].text) {
      return;
    }

    onClick(event, this.formatValue(newValue[0]), index);
  }

  public render(): JSX.Element[] {
    const { children, data, value } = this.props;

    return data.map((item, i) => children(item, i, value, this.onClick));
  }
}

export default Choice;