import * as React from 'react';
import * as classNames from 'classnames';
import ChoiceAction, { Base as ChoiceProps, Value, CallBackOnClick } from '../../action/Choice';
import { Props as IconTextProps } from '../../view/IconText';
import TabItem from '../../view/TabItem';

export interface Data {
  icon?: string;
  text: string;
  value?: string | number;
}

export interface Base extends ChoiceProps  {
  data: Data[];
  /** 单个选项的样式 */
  itemCls?: string;
  /** 选项被选中的样式 */
  activeCls?: string;
  /** 图标样式 */
  iconCls?: IconTextProps['iconCls'];
  /** 文本样式 */
  textCls?: IconTextProps['textCls'];
  /** 是否添加省略号 */
  ellipsis?: IconTextProps['ellipsis'];
}

interface Props extends Base {
  value: Value[] | undefined;
  onClick: CallBackOnClick;
}

function isActive(currentValue: Value[] | undefined, item: Data): boolean {
  if (currentValue) {
    const index = currentValue.slice().findIndex((itemValue) => {
      const value = item.value !== undefined ? item.value : item.text;
      const itemRealValue = itemValue.value !== undefined ? itemValue.value : itemValue.text;
      return itemRealValue.toString() === value.toString();
    });
    return index >= 0;
  }
  return false;
}

const Choice = (props: Props) => {
  const { data, value, onClick, multiple, itemCls, activeCls } = props;

  return (
    <ChoiceAction
      data={data}
      value={value}
      onClick={onClick}
      multiple={multiple}
    >
      {(item: Data, index, currentValue, tabOnClick) => (
        <TabItem
          {...props}
          key={item.text}
          className={classNames(itemCls, {
            [activeCls as string]: isActive(currentValue, item)
          })}
          icon={item.icon}
          text={item.text}
          onClick={(e, text) => tabOnClick(e, [item], index)}
        />
      )}
    </ChoiceAction>
  );
};

export default Choice;