import * as React from 'react';
import Icon from '../../../component/Icon';
import Text, { Props as TextProps } from '../../../component/Text';
import isSvgFile from '../../../utils/isSvgFile';

export interface Props {
  /** 图标的地址 */
  icon?: string;
  /** 文本的内容 */
  text: string;
  /** 图标样式 */
  iconCls?: string;
  /** 文本样式 */
  textCls?: string;
  ellipsis?: TextProps['ellipsis'];
}

// const prefixCls = 'yj-component-icontext';

class IconText extends React.PureComponent<Props> {
  public render(): (JSX.Element | null)[] {
    const { icon, text, ellipsis, iconCls, textCls } = this.props;

    return [
      icon ? (
        isSvgFile(icon) ? (
          <Icon key="icon" className={iconCls} src={icon} />
        ) : (
          <img key="icon" className={iconCls} src={icon} />
        )
      ) : null,
      <Text
        key="text"
        className={textCls}
        style={{
          whiteSpace: 'nowrap'
        }}
        ellipsis={ellipsis}
      >{text}</Text>
    ];
  }
}

export default IconText;
