import { observable, action } from 'mobx';
import Request from '../request';
import Base from '../../_base/datatype';

type List<T> = Array<T>;

class ObListMap<Key, Value> extends Base {
  constructor(request: Request) {
    super(request);
  }

  @observable protected value: Map<Key, List<Value>> = new Map();

  @action public set(key: Key, value: List<Value>): void {
    if (value.length > 0) {
      this.request.setState('complete');
    } else {
      this.request.setState('empty');
    }

    this.value.set(this.stringify(key), value);
  }

  public get(key: Key): List<Value> | undefined {
    return this.value.get(this.stringify(key));
  }

  public forEach(callback: (dataitem: List<Value>, key: Key) => void): void {
    this.value.forEach(callback);
  }
}

export default ObListMap;