import yj from '../typings';
import * as React from 'react';
import Behavior from '../model/behavior';
import global from './global';

export interface PageProps<Match> extends yj.PageProps<Match> {
}

interface InitParams {
  /** 页面实例 */
  self: Page<any, any, any>;
  /** 页面声明 */
  declaration: typeof Page;
  // /** 对外显示的类名，一般和页面的类名相同 */
  // displayName: string;
  // /** 页面名称，用于document.title */
  // pageName: string;
  /** 是否进行登录认证 */
  loginAuth?: boolean;
}

abstract class Page<Props, State, Match> extends React.Component<Props & PageProps<Match>, State> {
  constructor(props: Props & PageProps<Match>, state: State) {
    super(props, state);
  }
  /** 对外显示的类名，一般和页面的类名相同 */
  public static _displayName: string;
  /** Page实例 */
  public static _page: Page<yj.PageProps<any>, any, any>;
  /** 页面根容器的引用 */
  public static _pageRootElemRef: HTMLElement | null;
  /** 页面名称，用于document.title */
  public static _pageName: string | undefined;
  /** 是否进行登录认证 */
  public static _loginAuth: boolean = false;
  /** 进入页面的时间 */
  private entrytime: number = Date.now();
  /** 当获取这个属性时，得到已经停留了的时间 */
  public get _staytime(): number {
    const levaeTime = Date.now();
    return levaeTime - this.entrytime;
  }
  /** Page初始化时调用 */
  protected _init(params: InitParams): void {
    params.declaration._page = params.self;
    params.declaration._loginAuth = params.loginAuth || false;
    global.match = params.declaration._page.props.match;
  }
  /** 执行后退路由的操作时，会在动画完成后调用 */
  public static pageEnter: (() => void) | undefined;
  /** 执行push操作时，会在动画完成后调用上一个页面的pageLeave */
  public static pageLeave: (() => void) | undefined;
  // /** 页面行为数据里需要添加的参数 */
  // public static behaviorData: (() => TriggerParams['detailedData']) | undefined;
  // /** 页面行为数据里需要添加的参数 */
  // public static behaviorParams: () => OtherParams;

  /** 行为记录类实例 */
  public behavior: Behavior = new Behavior();
}

export default Page;