import yj from '../../typings';
import * as React from 'react';
import Account from '../../model/account';
import SubscribeMask from '../SubscribeMask';

export interface Props extends yj.ComponentProps {
  children?: any;
  /** 默认为component */
  type?: 'page' | 'component';
  onClick?: yj.OnClick<HTMLDivElement>;
  /** 关注弹出层的内容 */
  subscribeContent?: any;
}

interface State {
  /** 是否显示关注 */
  subscribe: boolean;
}

class Auth extends React.Component<Props, State> {
  constructor(props: Props, state: State) {
    super(props);

    this.onClick = this.onClick.bind(this);
    this.onClose = this.onClose.bind(this);
  }

  public state: State = {
    subscribe: false
  };

  public static defaultProps: Props = {
    type: 'component'
  };

  public componentDidMount(): void {
    const { type } = this.props;

    // 如果没有登录进入了这个页面，就跳转到登录页
    if (Account._loginState !== true && type === 'page') {
      Account.loginAuth();
    }
  }

  private onClick(e: React.MouseEvent<HTMLDivElement>): void {
    if (Account._subscribeState !== true) {
      this.setState({
        subscribe: true
      });
      return;
    }

    Account.loginAuth(() => {
      if (this.props.onClick) {
        this.props.onClick(e);
      }
    });
  }

  private onClose(e: React.MouseEvent<HTMLDivElement>): void {
    this.setState({
      subscribe: false
    });
  }

  public render(): JSX.Element[] | null {
    const { className, id, style, children, type, subscribeContent } = this.props;
    // 防止在未登录的情况下渲染页面
    return Account._loginState !== true && type === 'page' ? null : [
      <div
        key="children"
        className={className}
        id={id}
        style={style}
        onClick={this.onClick}
      >{children}</div>,
      <SubscribeMask
        key="SubscribeMask"
        visible={this.state.subscribe}
        onClose={this.onClose}
      >
        {subscribeContent}
      </SubscribeMask>
    ];
  }
}

export default Auth;