import * as React from 'react';
import * as classNames from 'classnames';
import Carousel, { Props as CarouselComponentProps } from '../Carousel';
import { BaseProps as StateBaseProps } from '../StateMachine';
import ImgView, { BaseProps as ImgViewProps } from '../ImgView';
import './index.scss';

export interface Data {
  id: string;
  src: string;
  link?: string;
}

export interface CarouselBaseProps extends CarouselComponentProps, StateBaseProps, ImgViewProps {
  itemCls?: string;
  imgCls?: string;
  /** 空视图 */
  emptyView?: any;
  /** 点击事件，会把当前的数据传入 */
  onClick?: (params: Data) => void;
}

export interface CarouselProps extends CarouselBaseProps  {
  /** 渲染的数据 */
  data: Data[];
}

interface State {
}

const prefixCls = 'yj-component-banner';
class AppCarousel extends React.Component<CarouselProps, State> {

  public shouldComponentUpdate(nextProps: CarouselProps, nextState: State): boolean {
    const dataChange = this.props.data !== nextProps.data;
    const slideIndexChange = this.props.index !== nextProps.index;
    const refreshIdChange = this.props.refreshId !== nextProps.refreshId;

    return dataChange || slideIndexChange || refreshIdChange;
  }

  private onClick = (data: Data) => {
    if (this.props.onClick) {
      this.props.onClick(data);
    }
  }

  public render(): JSX.Element | (JSX.Element | null)[] {
    const {
      data, dots, autoplay, onAfterSlide, shadow, index, loadingStyle, mode, emptyView, slideHeight, itemCls, imgCls, refreshId
    } = this.props;

    return (
      <Carousel
        loop
        autoplay={autoplay && data.length > 1}
        interval={3000}
        swiping={data.length > 1}
        dots={dots && data.length > 1}
        onAfterSlide={onAfterSlide}
        index={index}
        slideHeight={slideHeight}
        refreshId={refreshId}
      >
        {data.map((item) => {
          if (item.src) {
            return (
              <div
                key={item.id}
                className={itemCls}
              >
                <ImgView
                  className={classNames({
                    [`${prefixCls}-banner`]: true,
                    [imgCls || '']: !!imgCls
                  })}
                  onClick={() => this.onClick(item)}
                  completeView={item.src}
                  loadingStyle={loadingStyle}
                  shadow={shadow}
                  mode={mode}
                  stateMachine={false}
                />
              </div>
            );
          } else {
            return emptyView;
          }
        })}
      </Carousel>
    );
  }
}

export default AppCarousel;