import yj from '../../typings';
import { observer } from 'mobx-react';
import * as React from 'react';
import * as classNames from 'classnames';
import Carousel, { CarouselBaseProps, Data } from './Carousel';
import View from '../View';
import BannerService from './service';
// import Link from '../Link';
// import isHttp from '../../utils/isHttp';
// import Toast from '../Toast';
import PageStack from '../PageStack';
import './index.scss';

export interface BaseProps extends yj.ComponentProps, CarouselBaseProps {
  /** 是否显示图片当前索引/总数 */
  showIndex?: boolean;
  /** 加载时的样式 */
  loadingStyle?: React.CSSProperties;
}

// interface Props extends BaseProps, yj.ComponentBehavior {
//   /** 数据 */
//   data?: Data[];
//   model?: BannerModel;
//   className?: string;
//   /** 点击banner的行为记录 */
//   behaviorAction?: LoggerItem;
//   /** 滑动banner的行为记录 */
//   swipeBehaviorAction?: LoggerItem;
// }

interface Props extends BaseProps {
  data: Data[];
  service?: BannerService;
}

interface State {
  index: number;
}

const Complete = View.Complete;
const Empty = View.Empty;
const Fail = View.Fail;
const prefixCls = 'yj-component-banner';

@observer class Banner extends React.Component<Props, State> {
  constructor(props: Props, state: State) {
    super(props, state);
  }

  public state: State = {
    index: 0,
  };

  public shouldComponentUpdate(nextProps: Props, nextState: State): boolean {
    const stateIndexChange = this.state.index !== nextState.index;
    const propsIndexChange = this.props.index !== nextProps.index;
    const dataChange = this.props.data !== nextProps.data;
    const refreshIdChange = this.props.refreshId !== nextProps.refreshId;

    return stateIndexChange || propsIndexChange || dataChange || refreshIdChange;
  }

  private onAfterSlide = (index: number, direction: 'prev' | 'next', isAuto: boolean) => {
    const { onAfterSlide } = this.props;
    const stackComponent = PageStack.getStackComponent();
    const data = this.props.data;

    if (onAfterSlide) {
      onAfterSlide(index, direction, isAuto);
    }

    if (isAuto === false && stackComponent && stackComponent._page) {
      stackComponent._page.behavior.trigger({
        actionName: '滑动轮播图',
        detailedData: {
          url: data && data[index].src
        },
      });
    }

    this.setState({
      index
    });
  }

  private renderIndexBox(index: number, total: number): JSX.Element {
    return (
      <div className={`${prefixCls}-indexBox`} key="renderIndexBox">{index}&nbsp;/&nbsp;{total}</div>
    );
  }

  private onClick = (params: Data) => {
    const { onClick } = this.props;

    if (onClick) {
      onClick(params);
    }

    // if (behavior && behaviorAction) {
    //   behavior.trigger({
    //     detailedData: {
    //       ...behaviorData,
    //       url: params.link
    //     },
    //     ...behaviorAction
    //   });

    //   if (params.link && isHttp(params.link)) {
    //     behavior.postAction.send({
    //       callback: () => {
    //         Toast.close();
    //         Toast.show('正在跳转...');
    //         Link.go(params.link as string);
    //       }
    //     });
    //   }
    // } else {
    //   console.warn('跳转页面失败，此链接不是一个http协议的', 'link: ' + params.link);
    // }
  }

  public render(): JSX.Element | (JSX.Element | null)[] {
    const {
      dots, showIndex, autoplay, className, shadow, slideHeight, index, service, loadingStyle, style, mode, itemCls, imgCls,
      refreshId
    } = this.props;
    const data = this.props.data;

    const complete = (
      <div className={classNames(prefixCls, className)}>
        {showIndex && data.length > 0 ? this.renderIndexBox(this.state.index + 1, data.length) : null}
        <Carousel
          itemCls={itemCls}
          imgCls={imgCls}
          data={data}
          slideHeight={slideHeight}
          autoplay={autoplay && data.length > 1}
          dots={dots}
          onAfterSlide={this.onAfterSlide}
          shadow={shadow}
          index={index}
          loadingStyle={loadingStyle}
          mode={mode}
          onClick={this.onClick}
          refreshId={refreshId}
        />
      </div>
    );

    return service ? (
      <div className={prefixCls} style={style}>
        <View request={service} loadingStyle={loadingStyle}>
          <Complete>
            {complete}
          </Complete>
          <Empty>
            <div className={classNames(`${prefixCls}-empty`, className)} />
          </Empty>
          <Fail>
            <div className={classNames(`${prefixCls}-empty`, className)} />
          </Fail>
        </View>
      </div>
    ) : complete;
  }
}

export default Banner;