import ServiceRequest, { Props as RequestOptions  } from '../../app/request';
import Toast from '../../component/Toast';
import { Data } from '../../component/Banner/Carousel';
import ObArray from '../../app/datatype/obArray';

interface Props extends RequestOptions {
}

class Banner extends ServiceRequest {
  constructor(props: Props) {
    super(props);
  }

  public data: ObArray<Data> = new ObArray(this);
  public dataId: number = 0;

  /** 发送请求 */
  public async send(useCache?: boolean): Promise<void> {
    if (useCache === false) {
      this.request();
    } else {
      this.readCache<Data[]>({
        data: this.data.get().slice(),
        hit: (data) => this.data.set(data),
        miss: () => this.request()
      });
    }
  }

  /** 发送请求 */
  public async request(): Promise<void> {
    this.action({
      config: {
        url: this._api,
        method: 'get',
        params: this.requestData && this.requestData()
      }
    }).then((response) => {
      this.dataId++;
      if (response.retCode === '0000') {
        if (this._callback) {
          this._callback(response);
        }
      } else {
        this.data.set([]);
        Toast.show(response.retMsg);
      }
    }).catch((err) => {
      if (this._failback) {
        this._failback(err);
      }
    });
  }
}

export default Banner;