import yj from '../../typings';
import * as React from 'react';
import * as classNames from 'classnames';
import zellerFormula from '../../utils/zellerFormula';
import getDaysInDate from '../../utils/getDaysInDate';
import Day from './Day';
import Container from '../Container';
import './index.scss';

export interface Props extends yj.ComponentProps {
  /** 当前要渲染的日期 */
  date: Date;
  /** 激活的日子 */
  activeDate?: number[];
  /** 加点的日子  */
  dotDate?: number[];
  /** 添加到Day组件的类名 */
  dayCls?: string;
}

const prefixCls = 'yj-component-calendar-cell';

function include(days: number[], day: number): boolean {
  // 因为day的值是有序递增的，所以在循环时做了优化
  for (let i = day - 1; i < days.length; i++) {
    if (days[i] === day) {
      return true;
    }
  }

  return false;
}

const Cell = (props: Props) => {
  const { className, style, date, activeDate, dotDate, dayCls } = props;
  // 当月总天数
  const days = getDaysInDate(date);
  // 当月第一天是星期几
  const firstDay = zellerFormula(new Date(date.getFullYear(), date.getMonth(), 1));
  // 已渲染的天数
  let day: number = 0;
  let key: number = 0;

  return (
    <div
      className={classNames(prefixCls, className)}
      style={style}
    >
      {[0, 1, 2, 3, 4].map((col) => {
        return (
          <Container
            className={`${prefixCls}-col`}
            key={col}
            alignItems="center"
            justifyContent="center"
          >
            {[0, 1, 2, 3, 4, 5, 6].map((week) => {
              const visible = (col === 0 && week >= firstDay) || (col > 0 && day < days.length);
              day = visible ? day + 1 : day;
              key++;
              return (
                <Day
                  key={key}
                  className={dayCls}
                  visible={visible}
                  label={day}
                  weekend={week === 0 || week === 6}
                  active={activeDate ? include(activeDate, day) : false}
                  dot={dotDate ? include(dotDate, day) : false}
                />
              );
            })}
          </Container>
        );
      })}
    </div>
  );
};

export default Cell;