import yj from '../../typings';
import * as React from 'react';
import * as classNames from 'classnames';
import Carousel from '../Carousel';
import Cell from './Cell';
import dateAdd from '../../utils/dateAdd';
import rangeDateToDates from '../../utils/rangeDateToDates';
import './index.scss';

interface Props extends yj.ComponentProps {
  /** 最小日期，默认当前日期一年前 */
  minDate?: Date;
  /** 最大日期，默认当前日期一年后 */
  maxDate?: Date;
}

const prefixCls = 'yj-component-calendar';
const nowDate = new Date();

class Calendar extends React.Component<Props> {

  public static defaultProps: Props = {
    minDate: dateAdd(nowDate, -1, 'years'),
    maxDate: dateAdd(nowDate, 1, 'years')
  };

  /** 日期总数 */
  private dates: Date[] = rangeDateToDates(this.props.minDate as Date, this.props.maxDate as Date);

  public componentWillReceiveProps(nextProps: Props): void {
    const minDateChange = this.props.minDate !== nextProps.minDate;
    const maxDateChange = this.props.maxDate !== nextProps.maxDate;
    const nextMinDate = nextProps.minDate || dateAdd(nowDate, -1, 'years');
    const nextMaxDate = nextProps.maxDate || dateAdd(nowDate, 1, 'years');

    if (minDateChange || maxDateChange) {
      this.dates = rangeDateToDates(nextMinDate, nextMaxDate);
    }
  }

  /** 渲染Cell */
  private cellRender(): JSX.Element[] {
    const render = [];

    for (let i = 0; i < this.dates.length; i++) {
      const date = this.dates[i];
      render.push(
        <Cell
          key={date.getTime()}
          date={date}
        />
      );
    }

    return render;
  }

  public render(): JSX.Element {
    const { className, style } = this.props;

    return (
      <Carousel
        className={classNames(prefixCls, className)}
        style={style}
      >
        {this.cellRender()}
      </Carousel>
    );
  }
}

export default Calendar;