import * as React from 'react';
import { observer } from 'mobx-react';
import { Value, CallBackOnClick } from '../../_base/action/Choice/index';
import ChoiceBase, { Base as ChoiceProps } from '../../_base/component/Choice';
import Service from './service';

interface Props extends ChoiceProps {
  service?: Service;
  onClick?: CallBackOnClick;
}

@observer class Choice extends React.Component<Props> {
  constructor(props: Props) {
    super(props);

    this.onClick = this.onClick.bind(this);
  }

  private onClick(e: React.MouseEvent<HTMLDivElement>, value: Value[], index: number): void {
    const { service, onClick } = this.props;

    if (service) {
      service.changeValue(value);
    } else {
      console.warn('缺少service，value不会发生改变');
    }

    if (onClick) {
      onClick(e, value, index);
    }
  }

  public render(): JSX.Element {
    const { service } = this.props;
    const value = service && service.getValue();

    return (
      <ChoiceBase
        {...this.props}
        value={value}
        onClick={this.onClick}
      />
    );
  }
}

export default Choice;