import * as React from 'react';
import { Data } from '../Picker/Item';
import Picker, { Base as PikcerProps } from '../Picker';
import { SelectedItem } from '../Picker/Picker';
import Service from './service';
import PickerService from '../Picker/service';
import createRangeDate from '../../utils/createRangeDate';

export interface Props extends PikcerProps {
  service?: Service;
  minDate?: Date;
  maxDate?: Date;
  mode?: 'yyMM' | 'yyMMdd' | 'HHmm' | 'yyMMddHHmm';
}

class DatePicker extends React.Component<Props> {
  constructor(props: Props) {
    super(props);

    this.formatSelectedCol = this.formatSelectedCol.bind(this);
  }

  private pickerService: PickerService = new PickerService({
    fieldKey: 'dateChildrenToPicker',
    fieldName: '时间选择'
  });

  private data: Data[] = createRangeDate({
    minDate: this.props.minDate as Date,
    maxDate: this.props.maxDate as Date,
    mode: this.props.mode as Props['mode']
  });

  public static defaultProps: Props = {
    //value: new Date(),
    minDate: new Date(2000, 1, 1, 0, 0),
    maxDate: new Date(2030, 1, 1, 23, 0),
    mode: 'yyMMddHHmm'
  };

  private formatSelectedCol(currentCol: number, selectedItem: SelectedItem[]): number[] {
    return selectedItem as number[];
  }

  public render(): JSX.Element {

    return (
      <Picker
        {...this.props}
        data={this.data}
        cascade
        formatSelectedCol={this.formatSelectedCol}
        service={this.pickerService}
      />
    );
  }
}

export default DatePicker;