import yj from '../../typings/index';
import * as React from 'react';
import * as classNames from 'classnames';
import Fields from '../../_base/service/fields';
import FormService, { Params as SendParams } from './service';
import { Props as FormItemProps } from '../FormItem';

interface Props extends yj.ComponentProps  {
  service?: FormService;
  params?: SendParams;
  children?: any;
  onSubmit?: () => void;
}

interface State {

}

const prefixCls = 'yj-component-form';
class Form extends React.Component<Props, State> {
  constructor(props: Props, state: State) {
    super(props, state);

    this.onSubmit = this.onSubmit.bind(this);
  }

  public componentDidMount(): void {
    if (this.props.service) {
      this.props.service.fields = [];
    }
    this.findField(this.props.children);
  }

  public componentWillReceiveProps(nextProps: Props): void {
    if (this.props.children !== nextProps.children) {
      if (nextProps.service) {
        nextProps.service.fields = [];
      }
      this.findField(nextProps.children);
      if (nextProps.service) {
        nextProps.service.validate(undefined, false);
      }
    }
  }

  public findField(children: React.ReactChild): void {
    React.Children.forEach(children, (child) => {
      const formitem = child as React.ReactElement<FormItemProps>;
      if (formitem && formitem.props) {
        if (formitem.props.service) {
          this.pushField(formitem.props.service);
        } else if (formitem.props.children) {
          this.findField(formitem.props.children);
        }
      }
    });
  }

  private pushField(filed: Fields<any>): void {
    if (this.props.service) {
      this.props.service.fields.push(filed);
    }
  }

  private onSubmit(e: React.FormEvent<HTMLFormElement>): void {
    e.preventDefault();
    const service = this.props.service;

    if (service) {
      service.submit(this.props.params);
      if (!service.getComplete()) {
        return;
      }
    }

    if (this.props.onSubmit) {
      this.props.onSubmit();
    }
  }

  public render(): JSX.Element {
    const props = this.props;
    const { children } = props;
    return (
      <form
        className={classNames(prefixCls, props.className)}
        style={props.style}
        onSubmit={this.onSubmit}
        autoComplete="off"
      >
        {children}
      </form>
    );
  }
}

export default Form;
