import * as React from 'react';
import * as classNames from 'classnames';
import Container, { Props as ContainerProps } from '../Container';
import Icon from '../Icon';
import Text, { Props as TextProps } from '../Text';
import isSvgFile from '../../utils/isSvgFile';
import './index.scss';

interface Props extends TextProps {
  /** 图标的地址 */
  src: string;
  /** 文本的内容 */
  text: string;
  /** 图标样式 */
  iconCls?: string;
  /** 文本样式 */
  textCls?: string;
  /** 排列方向 */
  flexDirection?: ContainerProps['flexDirection'];
  /** 布局方式 */
  justifyContent?: ContainerProps['justifyContent'];
  /** 是否开启text元素flex=1的属性 */
  textFlexOne?: boolean;
}

const prefixCls = 'yj-component-icontext';
const IconText = (props: Props) => {
  const {
    src, justifyContent, onClick, style, text, ellipsis, ellipsisLines, iconCls, textCls, className, flexDirection, textFlexOne
  } = props;

  return (
    <Container
      className={classNames(prefixCls, className)}
      alignItems="center"
      justifyContent={justifyContent}
      flexDirection={flexDirection}
      onClick={onClick}
      style={style}
    >
      {isSvgFile(src) ? (
        <Icon className={classNames(`${prefixCls}-icon`, iconCls)} src={src} />
      ) : (
        <img className={classNames(`${prefixCls}-icon`, iconCls)} src={src} />
      )}
      <Container flex={textFlexOne ? 1 : undefined}>
        <Text
          className={classNames(`${prefixCls}-text`, textCls)}
          ellipsis={ellipsis}
          ellipsisLines={ellipsisLines}
        >{text}</Text>
      </Container>
    </Container>
  );
};

export default IconText;
