import yj from '../../typings';
import * as React from 'react';
import stringToHex from '../../utils/stringToHex';
import global from '../../app/global';
import Account from '../../model/account';
import isHttp from '../../utils/isHttp';
import pageLeave from '../../model/behavior/pageLeave';
import PageStack from '../PageStack';
import switchRouter from '../../utils/switchRouter';

export interface Props extends yj.ComponentProps {
  to: string;
  loginAuth?: boolean;
  children?: any;
}

/** 点击锁，防止多次点击 */
let lock = false;

/** 跳转到第三方链接 */
function toThirdLink(href: string): void {
  const urlsearchparams = new URLSearchParams(window.location.search);
  const pathname = window.location.hash.replace('#', '');
  const origin = window.location.origin;
  const locationpathname = window.location.pathname;
  // const encodePathname = pathname.indexOf('%2') >= 0 ? pathname : encodeURIComponent(pathname);
  const route = pathname.substr(0, 2) === ('2f' || '2F') ? pathname : stringToHex(pathname);

  // 设置当前的编码后的路由search
  urlsearchparams.set('route', route);

  const newUrl = origin + locationpathname + '?' + urlsearchparams.toString().replace(/%3F/g, '').replace(/%3f/g, '') + '#' + pathname;
  // 替换当前的路由
  window.history.replaceState({}, document.title, newUrl);
  // 跳转到目标地址
  window.location.href = href;
}

function go(url: string, replace: boolean): void {
  if (isHttp(url)) {
    toThirdLink(url);
  } else {
    if (replace) {
      global.history.replace(url);
    } else {
      global.history.push(url);
    }
  }
}

function link(url: string, replace: boolean, loginAuth?: boolean): void {
  if (lock === false) {
    if (loginAuth) {
      Account.loginAuth(() => {
        go(url, replace);
      });
    } else {
      go(url, replace);
    }
    lock = true;
    setTimeout(() => {
      lock = false;
    }, 320);
  }
}

class Link extends React.Component<Props> {

  /** push一个路由 */
  public static go(url: string, loginAuth?: boolean): void {
    link(url, false, loginAuth);
  }

  /** replace一个路由 */
  public static replace(url: string, loginAuth?: boolean): void {
    const componentPage = PageStack.getStackComponent();
    const route = switchRouter(global._routes, url);

    if (!componentPage._pageName) {
      console.warn(componentPage._displayName, '_pageNames属性为空');
    }

    if (!route) {
      console.error('路由错误');
    } else if (!route.props.component._pageName) {
      console.warn(route.props.component._displayName, '_pageNames属性为空');
    }

    pageLeave({
      behavior: componentPage._page.behavior,
      type: 'enter',
      now: componentPage._pageName || '',
      to: route && route.props.component._pageName || '',
      staytime: componentPage._page._staytime
    });
    link(url, true, loginAuth);
  }

  /** pop一个路由 */
  public static goBack(): void {
    global.history.goBack();
  }

  private onClick = () => {
    const { to, loginAuth } = this.props;
    Link.go(to, loginAuth);
  }

  public render(): JSX.Element {
    const { className, children, datatype } = this.props;
    return (
      <div
        className={className}
        onClick={this.onClick}
        data-type={datatype}
      >{children}</div>
    );
  }
}

export default Link;