import yj from '../../../typings';
import * as React from 'react';
import { observer } from 'mobx-react';
import * as classNames from 'classnames';
import { OnGeolocationParams } from '../service/geolocation';
import StateMachine, { BaseProps as StateBaseProps } from '../../StateMachine';
import { ChildrenProps } from '../index';

export interface Props extends yj.ComponentProps, ChildrenProps, StateBaseProps {
  onClick?: (e: React.MouseEvent<HTMLDivElement>) => void;
  /** 当定位完成时的回调 */
  onGeolocation?: OnGeolocationParams;
}

const prefixCls = 'yj-component-map-location';
@observer class Location extends React.Component<Props> {
  constructor(props: Props) {
    super(props);
  }

  public static displayName: string = 'Location';

  public componentDidMount(): void {
    const { service, onGeolocation } = this.props;

    if (service) {
      const geolocation = service.geolocation;

      if (onGeolocation) {
        geolocation.callback = onGeolocation.callback;
        geolocation.failback = onGeolocation.failback;

        if (onGeolocation.timeout) {
          geolocation.timeout = onGeolocation.timeout;
        }
      }

      if (geolocation.address === undefined) {
        geolocation.location();
      }
    }
  }

  public render(): JSX.Element | null {
    const { className, style, children, loadingStyle, onClick, service } = this.props;

    return children ? (
      <div
        className={classNames(prefixCls, className)}
        style={style}
        onClick={onClick}
      >
        <StateMachine
          state={service && service.geolocation.state || 'complete'}
          loadingStyle={loadingStyle}
        >{children}</StateMachine>
      </div>
    ) : null;
  }
}

export default Location;