import * as React from 'react';
import { observer } from 'mobx-react';
import * as classNames from 'classnames';
import Service from '../service';
import MarkerService from '../service/marker';
import Container from '../../Container';
import Popup from '../../Popup';
import Icon from '../../Icon';
import svgRootPath from '../../../utils/svgRootPath';
import Text from '../../Text';
import { ChildrenProps } from '../index';
const upSvg = svgRootPath + require('../../../assets/up.svg').id;
import './indx.scss';

interface Icon {
  src: string;
  size: [number, number];
}

export interface Props extends ChildrenProps {
  activeCls?: string;
}

interface State {
  visible: boolean;
}

const prefixCls = 'yj-component-map-markerlist';
@observer class MarkerList extends React.Component<Props, State> {
  constructor(props: Props) {
    super(props);

    this.onVisible = this.onVisible.bind(this);
    this.onListClick = this.onListClick.bind(this);
  }

  public state: State = {
    visible: false
  };

  private onVisible(): void {
    this.setState({
      visible: !this.state.visible
    });
  }

  private onListClick(marker: MarkerService): void {
    if (this.props.service) {
      this.props.service.setActiveMarker(marker);
      this.onVisible();
    }
  }

  public render(): JSX.Element | null {
    const { activeCls } = this.props;
    const service = this.props.service as Service;
    const choice = service.supporting.choice.getValue();
    const activeSupporting = choice && choice[0];
    const markers = activeSupporting && service.supportingMarker.get(activeSupporting.toString());

    return activeSupporting && markers ? (
      <Container className={`${prefixCls}-nearbyBlock`} alignItems="center" justifyContent="center">
        <Popup visible={this.state.visible} onClose={this.onVisible} closable>
          <Popup.Button onClick={this.onVisible}>
            <Container alignItems="center" justifyContent="center">
              <div>{activeSupporting ? `附近${activeSupporting}数量：${markers.length}个` : null}</div>
              {activeSupporting ? <Icon className={`${prefixCls}-nearbyBlock-up`} src={upSvg} /> : null}
            </Container>
          </Popup.Button>
          <Popup.Content>
            {activeSupporting ? (
              <div className={`${prefixCls}-popup`}>
                <Container className={`${prefixCls}-popup-title`} retinaBorder={{ direction: ['bottom'], color: '#e3e3e3' }}>
                  {`附近${activeSupporting}：${markers.length}个`}
                </Container>
                <div className={`${prefixCls}-popup-scroll`}>
                  {markers.map((marker) => (
                    <Container
                      key={marker.poi && marker.poi.point.lat.toString() + marker.poi.point.lng.toString()}
                      className={classNames(`${prefixCls}-popup-list`, {
                        [activeCls || '']: service.activeMarker === marker,
                        [`${prefixCls}-popup-active`]: service.activeMarker === marker
                      })}
                      alignItems="center"
                      onClick={() => this.onListClick(marker)}
                    >
                      <Container flex={1}><Text ellipsis>{marker.poi && marker.poi.title}</Text></Container>
                      <div>(距离约{marker.distance})</div>
                    </Container>
                  ))}
                </div>
              </div>
            ) : null}
          </Popup.Content>
        </Popup>
      </Container>
    ) : null;
  }
}

export default MarkerList;