import { observable, action } from 'mobx';
import Geolocation from './geolocation';
import Marker from './marker';
import Supporting from './supporting';

class MapModel {
  /** 百度地图对象 */
  private map: BMap.Map | undefined;
  /** 定位 */
  public geolocation: Geolocation = new Geolocation();
  /** 周边配套 */
  public supporting: Supporting = new Supporting();
  /** 周边配套标注  */
  @observable public supportingMarker: Map<string, Marker[]> = new Map();
  /** 中心点 */
  public center: Marker = new Marker();
  /** 激活的标注 */
  @observable public activeMarker: Marker | null = this.center;

  /** 初始化 */
  public init(map: BMap.Map): void {
    this.map = map;
  }

  /** 从定位点到地图中心点的距离，单位米，要在定位成功之后调用 */
  public getDistanceByCenter(): number {
    if (this.map) {
      const geolocation = this.geolocation.getValue();
      const end = this.center.mapPoint;

      if (geolocation && end) {
        return this.map.getDistance(geolocation.point, end);
      }
    }

    return 0;
  }

  @action public setActiveMarker(marker: Marker | null): void {
    this.activeMarker = marker;
  }
}

export default MapModel;