import { observable, action } from 'mobx';

class Marker {
  private map: BMap.Map | undefined;
  /** 半径圈 */
  public circle: BMap.Circle | undefined;
  /** 坐标点 */
  public mapPoint: BMap.Point | undefined;
  public poi: BMap.LocalResultPoi | undefined;
  public title: string = '';
  public address: string = '';
  /** 步行规划 */
  private walking?: BMap.WalkingRoute;
  /** 步行距离 */
  @observable public distance: string = '';

  public init(map: BMap.Map, point: BMap.Point, poi?: BMap.LocalResultPoi): void {
    this.map = map;
    this.mapPoint = point;
    this.poi = poi;
  }

  /** 获取缩放值 */
  public getZoom(radius?: number): number {
    if (this.map && radius) {
      const zoom = 16 - Math.floor((radius / 1000));
      // zoom最小不低于13
      return zoom < 13 ? 13 : zoom;
    }

    return 16;
  }

  public setCricle(radius: number, options?: BMap.CircleOptions): void {
    if (this.map && this.mapPoint) {
      if (this.circle) {
        this.map.removeOverlay(this.circle);
      } else {
        this.circle = new BMap.Circle(this.mapPoint, radius, {
          ...options,
          fillColor: (options && options.fillColor) || '#538701',
          strokeColor: (options && options.strokeColor) || '#538701',
          strokeWeight: (options && options.strokeWeight) || 1,
          fillOpacity: (options && options.fillOpacity) || 0.15,
          strokeOpacity: (options && options.strokeOpacity) || 0.15,
          enableMassClear: false
        });
      }

      this.map.setZoom(this.getZoom(radius));
      this.map.addOverlay(this.circle);
    } else {
      console.error('map或point为空');
    }
  }

  @action public setWalkingRoute(map: BMap.Map, orign: BMap.Point | [number, number], endPoint: BMap.Point): void {
    if (!this.walking) {
      const startPoint = Array.isArray(orign) ? new BMap.Point(orign[0], orign[1]) : orign;
      this.walking = new BMap.WalkingRoute(map, {
        onSearchComplete: (result) => {
          const plan = result.getPlan(0);
          if (plan) {
            this.setDistance(plan.getDistance() as string);
          }
        }
      });
      this.walking.search(startPoint, endPoint);
    }
  }

  @action private setDistance(distance: string): void {
    this.distance = distance;
  }
}

export default Marker;