import yj from '../../typings';
import * as React from 'react';
import * as classNames from 'classnames';
import './index.scss';

export interface Props {
  children: any;
  /** 关闭事件 */
  onClose?: yj.OnClick<HTMLDivElement>;
  className?: string;
  /** 是否允许点击背景关闭 */
  closable?: boolean;
  /** 背景样式 */
  maskCls?: string;
  /** 内容的样式 */
  contentCls?: string;
}

const prefixCls = 'yj-component-mask';
class Dialog extends React.Component<Props> {
  constructor(props: Props) {
    super(props);

    this.onClick = this.onClick.bind(this);
  }

  public componentDidMount(): void {
    document.body.style.overflow = 'hidden';
  }

  public componentWillUnmount(): void {
    document.body.style.overflow = '';
  }

  private onClick(e: React.MouseEvent<HTMLDivElement>): void {
    e.preventDefault();
    e.stopPropagation();

    if (this.props.closable && this.props.onClose) {
      this.props.onClose(e);
    }
  }

  private stopPropagation(e: React.MouseEvent<HTMLDivElement>): void {
    e.stopPropagation();
  }

  public render(): JSX.Element {
    const { children, maskCls, className, contentCls } = this.props;
    return (
      <div className={classNames(prefixCls, className)}>
        <div
          className={classNames(`${prefixCls}-bg`, maskCls)}
          onClick={this.onClick}
        />
        <div
          className={classNames(`${prefixCls}-content`, contentCls)}
          onClick={this.stopPropagation}
        >{children}</div>
      </div>
    );
  }
}

export default Dialog;