import Request, { Props as RequestProps } from '../../../app/request';
import Toast from '../../../component/Toast';
import ObArray from '../../../app/datatype/obArray';
import { Data } from '../Item';

export interface Props extends RequestProps {
}

class Get extends Request {
  constructor(props: Props) {
    super();

  }

  public data: ObArray<Data | Data[]> = new ObArray(this);

  /** 发送请求 */
  public async send(): Promise<void> {
    this.readCache<(Data | Data[])[]>({
      data: this.data.get().slice(),
      hit: (data) => this.data.set(data),
      miss: () => this.request()
    });
  }

  public async request(): Promise<void> {
    this.action({
      config: {
        url: this._api,
        method: 'post',
        data: this.requestData && this.requestData()
      }
    }).then((response) => {
      if (response.retCode === '0000') {
        if (this._callback) {
          this._callback(response);
        }
        this.data.set(response.result);
      } else {
        this.setState('fail');
        Toast.show(response.retMsg);
      }
    }).catch((err) => {
      console.error(err);
      Toast.show('网络繁忙');
    });
  }
}

export default Get;
