import yj from '../../typings';
import * as React from 'react';
import * as classNames from 'classnames';
import { observer } from 'mobx-react';
import Icon from '../Icon';
import Input from '../Input';
import Container from '../Container';
import Form from '../Form';
import FormItem from '../FormItem';
import Button from '../Button';
import FormService from '../Form/service';
import InputService from '../Input/service';
import PageStack from '../PageStack';
import { Event } from '../../app/request';
import svgRootPath from '../../utils/svgRootPath';
import './index.scss';

const svgFile = {
  search: svgRootPath + require('../../assets/search.svg').id
};

interface Props extends  yj.ComponentProps {
  inputService?: InputService;
  formService?: FormService;
  onSubmit?: (content: string | number | undefined) => void;
  placeholder?: string;
  params?: Event;
}
interface State {
  isFoucs: boolean;
}

const prefixCls = 'yj-component-search';
@observer class Search extends React.Component<Props> {

  public state: State = {
    isFoucs: false
  };

  public componentDidMount(): void {
    const { inputService, formService } = this.props;

    if (!inputService) {
      console.warn('Search组件缺少inputModel属性');
    }

    if (!formService) {
      console.warn('Search组件缺少formModel属性');
    }
  }

  private onFoucs = () => {
    this.setState({
      isFoucs: true
    });
  }

  private onBlur = () => {
    this.setState({
      isFoucs: false
    });
  }

  private onSubmit = () => {
    const { onSubmit, inputService } = this.props;
    const stackComponent = PageStack.getStackComponent();

    if (inputService) {
      if (onSubmit) {
        onSubmit(inputService.getValue());
      }

      stackComponent._page.behavior.trigger({
        actionName: '搜索',
        detailedData: { text: inputService.getValue() }
      });
    } else {
      console.warn('inputModel缺失onSubmit和behavior不会触发');
    }
  }

  public render(): JSX.Element {
    const { inputService, formService, placeholder, params, className, style } = this.props;
    return (
      <Container className={classNames(prefixCls, className)} style={style} alignItems="center">
        <Form className={`${prefixCls}-body`} service={formService} params={params} onSubmit={this.onSubmit}>
          <Icon className={`${prefixCls}-icon`} src={svgFile.search} />
          <FormItem className={`${prefixCls}-formitem`} service={inputService}>
            <FormItem.Body className={`${prefixCls}-formitem-body`}>
              <Input
                className={`${prefixCls}-input`}
                placeholder={placeholder}
                onFocus={this.onFoucs}
                onBlur={this.onBlur}
              />
            </FormItem.Body>
            <FormItem.After>
              <Button
                className={classNames(`${prefixCls}-text`, {
                  [`${prefixCls}-text-hide`]: !this.state.isFoucs
                })}
                htmlType="submit"
              >搜索</Button>
            </FormItem.After>
          </FormItem>
        </Form>
      </Container>
    );
  }
}

export default Search;
