import yj from '../../typings';
import * as React from 'react';
import * as classNames from 'classnames';
import isPassiveSupported from '../../utils/isPassiveSupported';
import './index.scss';
// const throttle: lodash.LoDashStatic['throttle'] = require('lodash/throttle');

interface Props extends yj.ComponentProps {
  /** 阈值 */
  top: number;
  /** 状态改变时调用，isOver: 是否超过设置的top值 */
  onChange: (isOver: boolean) => void;
  children?: any;
}
interface State {
}

const prefixCls = 'yj-component-sticky';
class Sticky extends React.Component<Props, State> {
  constructor(props: Props, state: State) {
    super(props, state);
  }

  /** 是否超过设置的top值 */
  private isOver: boolean | undefined;
  private elemRef!: HTMLDivElement;

  /** 设置是否触发粘性 */
  private setSticky = () => {
    const top = window.scrollY || this.elemRef.scrollTop;
    const isOver = top >= this.props.top;

    // 判断状态是否发生变化
    if (isOver !== this.isOver) {
      this.props.onChange(isOver);
    }

    this.isOver = isOver;
  }

  /** 限制stateRender的频率 */
  // private throttled: (() => void) & _.Cancelable = throttle(this.setSticky, 50, { leading: false });

  public componentDidMount(): void {
    // 初始化时调用一次，防止初始化时滚动条不在顶部
    this.setSticky();
    // this.elemRef.addEventListener('touchmove', this.throttled, isPassiveSupported ? { passive: true } : false);
    this.elemRef.addEventListener('scroll', this.setSticky, isPassiveSupported ? { passive: true } : false);
  }

  public componentWillUnmount(): void {
    if (this.elemRef) {
      this.elemRef.removeEventListener('touchmove', this.setSticky);
      this.elemRef.removeEventListener('touchend', this.setSticky);
    }
  }

  private saveElemRef = (el: HTMLDivElement) => {
    this.elemRef = el;
  }

  public render(): JSX.Element {
    const { className, children, style } = this.props;

    return (
      <div
        className={classNames(prefixCls, className)}
        ref={this.saveElemRef}
        style={style}
      >
        {children}
      </div>
    );
  }
}

export default Sticky;
