import * as React from 'react';
import { observer } from 'mobx-react';
import { Value, CallBackOnClick } from '../../_base/action/Choice';
import ChoiceBase, { Base as ChoiceProps } from '../../_base/component/Choice';

export interface Props extends ChoiceProps {
  value?: Value;
  onClick?: CallBackOnClick;
}

interface State {
  value: Value;
}

@observer class Tab extends React.Component<Props> {
  constructor(props: Props) {
    super(props);

    this.onClick = this.onClick.bind(this);
  }

  public state: State = {
    value: this.props.value ? this.props.data[this.getIndex()] : this.props.data[0]
  };

  private getIndex(): number {
    return this.props.data.findIndex((value) => this.props.value ? this.props.value.text === value.text : false);
  }

  private onClick(e: React.MouseEvent<HTMLDivElement>, value: Value[], index: number): void {
    const { onClick } = this.props;

    if (onClick) {
      onClick(e, value, index);
    }

    this.setState({
      value: value[0]
    });
  }

  public render(): JSX.Element {
    const value = this.state.value;

    return (
      <ChoiceBase
        {...this.props}
        value={[value]}
        onClick={this.onClick}
        multiple={false}
      />
    );
  }
}

export default Tab;