import ServiceRequest, { Props } from '../../app/request';
import Service, { ExtendsRequestProps } from './index';
import Toast from '../../component/Toast';

class Verification extends ServiceRequest {
  constructor(service: Service, props: Props & ExtendsRequestProps) {
    super(props);

    this.service = service;
  }

  private service: Service;
  /** 是否可以发送的状态 */
  public get completeState(): boolean {
    return this.service.tel.getError() === false;
  }

  /** 发送请求 */
  public async send(): Promise<void> {
    if (this.completeState) {
      this.action({
        config: {
          url: this._api,
          method: 'post',
          data: this.requestData && this.requestData()
        }
      }).then((response) => {
        if (response.retCode === '0000' && this._callback) {
          this._callback(response);
        } else {
          Toast.show(response.retMsg);
        }
      }).catch((err) => {
        if (this._failback) {
          this._failback(err);
        }
        Toast.show('网络繁忙');
      });
    } else {
      Toast.show('请输入正确的手机号');
    }
  }
}

export default Verification;
