import dateformat from '../../utils/dateformat';
import device from '../../utils/device';
import env from '../../utils/env';
import { Params } from '../../app/request';
import PostRequest from './post';
import Account from '../account';
import PageStack from '../../component/PageStack';
import Page from '../../app/page';

interface BaseAction {
  /** 行为名称 */
  actionName: string;
}

// export interface LoggerItem {
//   /** 序号 */
//   id: number;
//   /** 动作名称 */
//   actionName: string;
//   /** 上一个动作id，只有是页面类型时才有；因为上一个动作可能来自不同的页面，所以会从数组中去匹配 */
//   previousActionIds?: number[];
//   /** 只有是页面类型的行为记录是才有 */
//   displayName?: string;
//   /** 是否要覆盖在上一个动作ID，displayName存在时默认为true */
//   previous?: boolean;
// }

// export type Logger<Id extends string> = {
//   [id in Id]: LoggerItem;
// };

export interface Action extends BaseAction {
  /** 停留时间 */
  stopSeconds?: number;
  /** 创建时间 */
  createTime: string;
  detailedData: string;
  /** 用户设备信息 */
  userAgent: Navigator['userAgent'];
}

export interface TriggerParams extends BaseAction {
  /** 停留时间 */
  stopSeconds?: number;
  /** 详细记录数据 */
  detailedData?: {
    [key: string]: string | number | undefined;
  };
  /** 页面名称 */
  pageName?: string;
  /** 其他参数 */
  other?: Params;
}

interface Options {
  /** 用于发送行为记录 */
  api: string;
  /** account.data里有发送行为记录所需要的参数 */
  account: Account;
  /** 默认参数 */
  defaultParams?: Params;
}

class Behavior {
  public actions: Action[] = [];
  public postAction: PostRequest = new PostRequest(this);

  /** 配置参数 */
  public static options: Options | undefined;

  /** 触发动作 */
  public trigger = (params: TriggerParams) => {
    if (Behavior.options) {
      const { defaultParams } = Behavior.options;
      const { actionName, detailedData, stopSeconds, other, pageName } = params;
      const componentPageName = pageName || (PageStack.getStackComponent() as typeof Page)._pageName;

      const action = {
        actionName: componentPageName + '-' + actionName,
        detailedData: JSON.stringify(detailedData || {}),
        createTime: dateformat(new Date(), 'yyMMddHHmmss'),
        stopSeconds,
        userAgent: device.userAgent,
        ...other,
        ...defaultParams
      };

      this.actions.push(action);
      if (env === 'development') {
        console.log(action);
      }
    }
  }
}

export default Behavior;