type Unit = 'years' | 'months' | 'weeks' | 'days' | 'hours' | 'minutes' | 'seconds';

/**
 * 对日期做加减法（value可以为负数）
 * @param date 需要做计算的日期
 * @param value 需要加减的值
 * @param unit 时间单位
 */
export default function dateAdd(date: Date, value: number, unit: Unit): Date {
  // 参数date的复制，防止改变参数的原值
  const temp = new Date(date.getTime());
  // 当前时间戳
  const timestamp = date.getTime();
  // 1秒
  const seconds = 1000;
  // 1分钟
  const minutes = 60 * seconds;
  // 1小时
  const hours = 60 * minutes;
  // 1天
  const days = 24 * hours;
  // 1周
  const weeks = 7 * days;
  // 当前date年份
  const nowYear = temp.getFullYear();
  // 当前date月份
  const nowMonth = temp.getMonth();
  // 目标日期时间戳
  let targetTimestamp: number = 0;

  switch (unit) {
    case 'years':
      targetTimestamp = temp.setFullYear(nowYear + value);
      break;
    case 'months':
      targetTimestamp = temp.setMonth(nowMonth + value);
      break;
    case 'days':
      targetTimestamp = timestamp + value * days;
      break;
    case 'weeks':
      targetTimestamp = timestamp + value * weeks;
      break;
    case 'hours':
      targetTimestamp = timestamp + value * hours;
      break;
    case 'minutes':
      targetTimestamp = timestamp + value * minutes;
      break;
    case 'seconds':
      targetTimestamp = timestamp + value * seconds;
      break;
    default:
      break;
  }

  return new Date(targetTimestamp);
}