export interface BaseParams {
  /** 是否开启循环 */
  loop?: boolean;
  /** 子元素排列的顺序 */
  index: number;
}

interface Params extends BaseParams {
  /** slide的总宽度 */
  frameWidth: number;
  /** 单个slide的宽度 */
  slideWidth: number;
  /** 当前position的值 */
  positionValue: number;
  /** slide个数 */
  slideCount: number;
  /** 偏移值 */
  offset?: number;
}

/** 获取slide目标的Position位置 */
export default function getSlideTargetPosition(params: Params): number {
  const { frameWidth, slideWidth, index, loop, positionValue, slideCount } = params;
  const offset = params.offset || 0;
  // 显示第几个slide
  const slidesToShow = frameWidth / slideWidth;
  /** 目前的位置 */
  const targetPosition = slideWidth * index + offset;
  /** position尾端的位置 */
  const end = slideWidth * slidesToShow * -1;

  // 开启循环
  if (loop) {
    const slidesBefore = Math.ceil(positionValue / slideWidth);

    if (slideCount - slidesBefore <= index) {
      return slideWidth * (slideCount - index) * -1;
    }

    let slidesAfter = Math.ceil((Math.abs(positionValue) - Math.abs(end)) / slideWidth);

    if (slideWidth !== 1) {
      slidesAfter = Math.ceil((Math.abs(positionValue) - slideWidth) / slideWidth);
    }

    if (index <= slidesAfter - 1) {
      return slideWidth * (slideCount + index);
    }
  }
  return targetPosition;
}